<?php

use IpagWoocommerce\Credentials;
use IpagWoocommerce\Environment;
use IpagWoocommerce\Transaction;

abstract class WC_iPag_Loader extends WC_Payment_Gateway
{
    public $identification;
    public $environment;

    /**
     * Recupera o IP do Cliente
     *
     * @return string
     */
    public function getClientIp($onlyipv4 = false)
    {
        $ip = '';
        if (array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)) {
            $ip = $_SERVER["HTTP_X_FORWARDED_FOR"];
        } else if (array_key_exists('REMOTE_ADDR', $_SERVER)) {
            $ip = $_SERVER["REMOTE_ADDR"];
        } else if (array_key_exists('HTTP_CLIENT_IP', $_SERVER)) {
            $ip = $_SERVER["HTTP_CLIENT_IP"];
        }
        if ($onlyipv4) {
            if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
                return $ip;
            } elseif (filter_var($_SERVER['SERVER_ADDR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
                return $_SERVER['SERVER_ADDR'];
            } else {
                $host = gethostname();
                $ip = gethostbyname($host);
                return $ip;
            }
        }

        return '';
    }

    public function getDescricaoPedido($order)
    {
        $cartItems = $order->get_items();
        $itemsData = array();
        $json = array();
        $i = 0;
        foreach ($cartItems as $item) {
            $itemsData["quant"] = $item['qty'];
            $itemsData["descr"] = preg_replace('/[<>\-&%\/]/', '', $item['name']);
            $itemsData["valor"] = number_format($item['line_subtotal'] / $item['qty'], 2, '.', '');
            $itemsData["id"] = $item['product_id'];
            $json[++$i] = $itemsData;
        }
        return json_encode($json);
    }

    public function valorParcelas($total, $parcelas, $juros)
    {
        $vparcelas = array();
        $denominador = 0;
        for ($p = 1; $p <= $parcelas; $p++) {
            $denominador += 1 / pow(1 + ($juros / 100), $p);
            $value = round($total / $denominador, 2);
            array_push($vparcelas, $value);
        }

        return $vparcelas;
    }

    public function order_info()
    {
        if (!empty($_REQUEST['post'])):
            $order_id = $_REQUEST['post'];
            $order = new WC_Order($order_id);
            $method = is_callable(array($order, 'get_payment_method')) ? $order->get_payment_method() : $order->payment_method;

            $t_id = get_post_meta($_REQUEST['post'], '_transaction_id', true);
            $t_msg = get_post_meta($_REQUEST['post'], '_transaction_message', true);
            $o_msg = get_post_meta($_REQUEST['post'], '_operator_message', true);
            $n_parcelas = get_post_meta($_REQUEST['post'], '_installment_number', true);
            $bandeira = get_post_meta($_REQUEST['post'], '_card_type', true);
            $bandeira2 = get_post_meta($_REQUEST['post'], '_card_type_second_card', true);
            $status = get_post_meta($_REQUEST['post'], '_status_payment', true);
            $billet_url = get_post_meta($_REQUEST['post'], '_billet_url', true);
            $lp_sub = get_post_meta($_REQUEST['post'], '_last_payment_sub', true);

            $ipagMethods = array('ipag-gateway_boleto', 'ipag-gateway-double-card', 'ipag-gateway', 'ipag-gateway_debito', 'ipag-gateway_itaushopline', 'ipag-gateway_pix');
            if(in_array($method, $ipagMethods)) :
            ?>
                    <h4><?php _e('Payment Details', 'ipag-gateway');?></h4>
                    <div class="payment">
                        <p>
                            <strong><?php _e('Transaction ID:', 'ipag-gateway');?></strong><br>
                            <?php echo '<span style="word-wrap:break-word;">'.$t_id.'</span>'; ?>
                            <br>
                            <strong><?php _e('Transaction Message:', 'ipag-gateway');?></strong><br>
                            <?php echo '<span id="t_msg">'.$t_msg.'</span>'; ?>
                            <br>
                            <strong><?php _e('Operator Message:', 'ipag-gateway');?></strong><br>
                            <?php echo '<span id="t_msg">'.$o_msg.'</span>'; ?>
                            <br>
                            <?php if ($n_parcelas): ?>
                                <strong><?php _e('Installments Number:', 'ipag-gateway');?></strong><br>
                                <?php echo '<span>'.$n_parcelas.'</span>'; ?>
                                <br>
                            <?php endif;?>
<?php if ($bandeira): ?>
                            <strong><?php _e('Card Brand:', 'ipag-gateway');?></strong><br>
                            <?php echo '<span>'.$this->getCardBrand($bandeira).'</span>'; ?>
                            <br>
                        <?php endif;?>
                    </p>
                    <?php if ($lp_sub): ?>
                        <h4><?php _e('Subscription', 'ipag-gateway');?></h4>
                        <p>
                            <strong><?php _e('Last Payment:', 'ipag-gateway');?></strong><br>
                            <?php echo '<span>'.$lp_sub.'</span>'; ?>
                        </p>
                    <?php endif;?>
<?php if ($status == 5 && $bandeira && empty($bandeira2)): ?>
                        <input type="button" id="capt_payment" value="<?php _e('Capture', 'ipag-gateway')?>" class="button button-primary" style="margin-top:10px;"/>
                    <?php elseif ($status == 5 && $bandeira): ?>
                        <input type="button" id="capt_payment_first" value="<?php _e('Capture', 'ipag-gateway')?>" class="button button-primary" style="margin-top:10px;"/>
                    <?php endif;?>
<?php if ($billet_url && !(strpos($method, 'pix') !== false)): ?>
                        <a id="billet_url" target="_blank" href="<?php echo $billet_url; ?>" class="button button-primary" style="margin-top:10px;"><?php _e('Imprimir Boleto', 'ipag-gateway')?></a>
                    <?php endif;?>
                    <?php if ($billet_url && strpos($method, 'pix') !== false): ?>
                        <a id="pix_url" target="_blank" href="<?php echo $billet_url; ?>" class="button button-primary" style="margin-top:10px;"><?php _e('Visualizar QRCode', 'ipag-gateway')?></a>
                    <?php endif;?>
<?php if ($t_id): ?>
                        <input type="button" id="consult_transaction" value="<?php _e('Consult', 'ipag-gateway')?>" class="button button-primary" style="margin-top:10px;"/>
                    <?php else: ?>
                        <p>Não foi possível recuperar a identificação da transação, por favor sincronize a transação pelo painel do iPag.</p>
                    <?php endif;?>
                </div>
            <?php endif;
            endif;
    }

    public function orderInfoSecondCard()
    {
        if (!empty($_REQUEST['post'])):
            $t_id = get_post_meta($_REQUEST['post'], '_transaction_id_second_card', true);
            $t_msg = get_post_meta($_REQUEST['post'], '_transaction_message_second_card', true);
            $o_msg = get_post_meta($_REQUEST['post'], '_operator_message_second_card', true);
            $n_parcelas = get_post_meta($_REQUEST['post'], '_installment_number_second_card', true);
            $bandeira = get_post_meta($_REQUEST['post'], '_card_type_second_card', true);
            $status = get_post_meta($_REQUEST['post'], '_status_payment_second_card', true);
            $billet_url = get_post_meta($_REQUEST['post'], '_billet_url_second_card', true);
            $lp_sub = get_post_meta($_REQUEST['post'], '_last_payment_sub_second_card', true);
            if (!empty($t_id)):
            ?>
                            <h4><?php _e('Payment Details', 'ipag-gateway');?></h4>
                            <div class="payment">
                                <p>
                                    <strong><?php _e('Transaction ID:', 'ipag-gateway');?></strong><br>
                                    <?php echo '<span style="word-wrap:break-word;">'.$t_id.'</span>'; ?>
                                    <br>
                                    <strong><?php _e('Transaction Message:', 'ipag-gateway');?></strong><br>
                                    <?php echo '<span id="t_msg_second">'.$t_msg.'</span>'; ?>
                                    <br>
                                    <strong><?php _e('Operator Message:', 'ipag-gateway');?></strong><br>
                                    <?php echo '<span id="t_msg_second">'.$o_msg.'</span>'; ?>
                                    <br>
                                    <?php if ($n_parcelas): ?>
                                        <strong><?php _e('Installments Number:', 'ipag-gateway');?></strong><br>
                                        <?php echo '<span>'.$n_parcelas.'</span>'; ?>
                                        <br>
                                    <?php endif;?>
<?php if ($bandeira): ?>
                                <strong><?php _e('Card Brand:', 'ipag-gateway');?></strong><br>
                                <?php echo '<span>'.$this->getCardBrand($bandeira).'</span>'; ?>
                                <br>
                            <?php endif;?>
                        </p>
                        <?php if ($lp_sub): ?>
                            <h4><?php _e('Subscription', 'ipag-gateway');?></h4>
                            <p>
                                <strong><?php _e('Last Payment:', 'ipag-gateway');?></strong><br>
                                <?php echo '<span>'.$lp_sub.'</span>'; ?>
                            </p>
                        <?php endif;?>
<?php if ($status == 5 && $bandeira): ?>
                            <input type="button" id="capt_payment_second" value="<?php _e('Capture', 'ipag-gateway')?>" class="button button-primary" style="margin-top:10px;"/>
                        <?php endif;?>
<?php if ($billet_url): ?>
                            <a id="billet_url" target="_blank" href="<?php echo $billet_url; ?>" class="button button-primary" style="margin-top:10px;"><?php _e('Imprimir Boleto', 'ipag-gateway')?></a>
                        <?php endif;?>
<?php if ($t_id): ?>
                            <input type="button" id="consult_transaction_second" value="<?php _e('Consult', 'ipag-gateway')?>" class="button button-primary" style="margin-top:10px;"/>
                        <?php else: ?>
                            <p>Não foi possível recuperar a identificação da transação, por favor sincronize a transação pelo painel do iPag.</p>
                        <?php endif;?>
                    </div>
                <?php endif;endif;
    }

    public function writeLog($msg, $force = false, $suffix = '')
    {
        if ($force || $this->debug == 'yes') {
            if (!empty($suffix)) {
                $suffix = '-'.$suffix;
            }
            $paymentLabel = 'iPag';
            $dir = plugin_dir_path(__DIR__);
            $nome = date('Y-m-d')."-".$paymentLabel."Log".$suffix.".txt";
            if (!file_exists($dir.'logs/')) {
                $created = mkdir($dir.'logs', 0777);
            }
            if (file_exists($dir.'logs/') && is_writable($dir.'logs/')) {
                $log = fopen($dir.'logs/'.$nome, 'a');
                fwrite($log, date('Y-m-d H:i:s').': '.PHP_EOL);
                fwrite($log, $msg.PHP_EOL);
                fclose($log);
                return true;
            } else {
                return false;
            }
        }
    }

    public function get_status($id)
    {
        $id = (int) $id;
        $statuses = array(
            1 => __('Transaction Initiated', 'ipag-gateway'),
            2 => __('Printed Billet', 'ipag-gateway'),
            3 => __('Transaction Canceled', 'ipag-gateway'),
            4 => __('Transaction In Analysis', 'ipag-gateway'),
            5 => __('Transaction Approved', 'ipag-gateway'),
            6 => __('Transaction Approved Partial Value', 'ipag-gateway'),
            7 => __('Transaction Refused', 'ipag-gateway'),
            8 => __('Transaction Approved & Captured', 'ipag-gateway'),
        );

        return $statuses[$id];
    }

    public function getOrderData($order)
    {
        $args = array();
        // WooCommerce 3.0 or later.
        if (method_exists($order, 'get_meta')) {
            $args['billingAddress'] = $order->get_billing_address_1();
            $args['billingNumber'] = $order->get_meta('_billing_number');
            if (empty($args['billingNumber'])) {
                $args['billingNumber'] = preg_replace('/\D/', '', $args['billingAddress']);
            }
            $cpf = $order->get_meta('_billing_cpf');
            $cnpj = $order->get_meta('_billing_cnpj');

            // Get the user ID from an Order ID
            $user_id = $order->get_customer_id();
            if ($user_id) {
                $customer = new WC_Customer( $user_id );
                if ($customer && empty($cpf) && empty($cnpj)) {
                    $cpf = get_user_meta($user_id, 'billing_cpf', true);
                    $cnpj = get_user_meta($user_id, 'billing_cnpj', true);
                }
            }

            $documento = empty($cpf) ? $cnpj : $cpf;
            $args['userId'] = $order->get_user_id();
            $args['documento'] = $documento;
            $args['billingName'] = $order->get_billing_first_name().' '.$order->get_billing_last_name();
            $args['billingEmail'] = $order->get_billing_email();
            $args['billingPhone'] = $order->get_billing_phone();
            $args['billingCellphone'] = $order->get_meta('_billing_cellphone');
            $args['billingNeighborhood'] = $order->get_meta('_billing_neighborhood');
            $args['billingAddress2'] = $order->get_billing_address_2();
            $args['billingCity'] = $order->get_billing_city();
            $args['billingState'] = $order->get_billing_state();
            $args['billingPostcode'] = $order->get_billing_postcode();
            $args['billingBirthdate'] = $order->get_meta('_billing_birthdate');
            $args['billingSex'] = $order->get_meta('_billing_sex');

            // Shipping fields.
            $args['shippingAddress'] = $order->get_shipping_address_1();
            $args['shippingNumber'] = $order->get_meta('_shipping_number');
            if (empty($args['shippingNumber'])) {
                $args['shippingNumber'] = preg_replace('/\D/', '', $args['shippingAddress']);
            }
            $args['shippingNeighborhood'] = $order->get_meta('_shipping_neighborhood');
            $args['shippingAddress2'] = $order->get_shipping_address_2();
            $args['shippingPostcode'] = $order->get_shipping_postcode();
            $args['shippingCity'] = $order->get_shipping_city();
            $args['shippingState'] = $order->get_shipping_state();
        } else {
            $args['billingAddress'] = $order->billing_address_1;
            if (!empty($order->billing_number)) {
                $args['billingNumber'] = $order->billing_number;
            } else {
                $args['billingNumber'] = preg_replace('/\D/', '', $order->billing_address_1);
            }
            $cpf = $order->billing_cpf;
            $cnpj = $order->billing_cnpj;
            $documento = empty($cpf) ? $cnpj : $cpf;
            $args['userId'] = $order->user_id;
            $args['documento'] = $documento;
            $args['billingName'] = $order->billing_first_name.' '.$order->billing_last_name;
            $args['billingEmail'] = $order->billing_email;
            $args['billingPhone'] = $order->billing_phone;
            $args['billingCellphone'] = $order->billing_cellphone;
            $args['billingNeighborhood'] = $order->billing_neighborhood;
            $args['billingAddress2'] = $order->billing_address_2;
            $args['billingCity'] = $order->billing_city;
            $args['billingState'] = $order->billing_state;
            $args['billingPostcode'] = $order->billing_postcode;
            $args['billingBirthdate'] = $order->billing_birthdate;
            $args['billingSex'] = $order->billing_sex;

            $args['shippingAddress'] = $order->shipping_address_1;
            if (!empty($order->shipping_number)) {
                $args['shippingNumber'] = $order->shipping_number;
            } else {
                $args['shippingNumber'] = preg_replace('/\D/', '', $order->shipping_address_1);
            }
            $args['shippingNeighborhood'] = $order->shipping_neighborhood;
            $args['shippingAddress2'] = $order->shipping_address_2;
            $args['shippingPostcode'] = $order->shipping_postcode;
            $args['shippingCity'] = $order->shipping_city;
            $args['shippingState'] = $order->shipping_state;
        }
        $args['billingNumber'] = preg_replace('/[\D]/', '', $args['billingNumber']);
        $args['billingPhone'] = preg_replace('/[\D]/', '', $args['billingPhone']);
        if (strlen($args['billingNumber']) > 5) {
            $args['billingNumber'] = substr($args['billingNumber'], 0, 5);
        }
        if (empty($args['billingNumber'])) {
            $args['billingNumber'] = '00';
        }
        return $args;
    }

    public function getCardType($ccNum)
    {
        if (preg_match("/^(401178|401179|431274|438935|451416|457393|457631|457632|504175|"
            ."627780|636297|636368|(506699|5067[0-6]\d|50677[0-8])|"
            ."(50900\d|5090[1-9]\d|509[1-9]\d{2})|65003[1-3]|(65003[5-9]|65004\d|65005[0-1])|"
            ."(65040[5-9]|6504[1-3]\d)|(65048[5-9]|65049\d|6505[0-2]\d|65053[0-8])|"
            ."(65054[1-9]|6505[5-8]\d|65059[0-8])|(65070\d|65071[0-8])|65072[0-7]|"
            ."(65090[1-9]|65091\d|650920)|(65165[2-9]|6516[6-7]\d)|(65500\d|65501\d)|"
            ."(65502[1-9]|6550[3-4]\d|65505[0-8]))[0-9]{10,12}$/i", $ccNum)) {
            return "elo";
        }

        if (preg_match("/^(((606282)\d{0,10})|((3841)\d{0,12}))$/i", $ccNum)) {
            return "hipercard";
        }

        if (preg_match("/^(637095|637612|637599|637609|637600|637568)[0-9]{10}$/i", $ccNum)) {
            return "hiper";
        }

        if (preg_match("/^[5|2][1-5][0-9]{14}$/i", $ccNum)) {
            return "mastercard";
        }

        if (preg_match("/^4[0-9]{12}([0-9]{3})?$/i", $ccNum)) {
            return "visa";
        }

        if (preg_match("/^3[47][0-9]{13}$/i", $ccNum)) {
            return "amex";
        }

        if (preg_match("/^3(0[0-5]|[68][0-9])[0-9]{11}$/i", $ccNum)) {
            return "diners";
        }

        if (preg_match("/^6011[0-9]{12}$/i", $ccNum)) {
            return "discover";
        }

        if (preg_match("/^(?:2131|1800|35\d{3})\d{11}$/i", $ccNum)) {
            return "jcb";
        }

    }

    public function getCardBrand($cb_id)
    {
        switch ($cb_id) {
            case '1':
                return 'Diners';
                break;
            case '2':
                return 'Mastercard';
                break;
            case '3':
                return 'Visa';
                break;
            case '5':
                return 'Amex';
                break;
            case '6':
                return 'HiperCard';
                break;
            case '7':
                return 'Aura';
                break;
            case '8':
                return 'Carrefour';
                break;
        }
    }

    public static function updateTransaction($trans_id, $status_pagamento)
    {
        global $wpdb;
        $wpdb->update($wpdb->prefix.'ipag_gateway', array('status' => $status_pagamento), array('trans_id' => $trans_id));
    }

    public function registerTransaction($trans_id, $order_id)
    {
        global $wpdb;
        $date = date('Y-m-d');
        return $wpdb->insert($wpdb->prefix.'ipag_gateway', array('order_id' => $order_id, 'trans_id' => $trans_id, 'payment_date' => $date));
    }

    public function existsTransaction($transid)
    {
        global $wpdb;
        $table_name = $wpdb->prefix.'ipag_gateway';
        $row = $wpdb->get_row('SELECT * FROM '.$table_name.' WHERE trans_id = "'.$transid.'"');
        if (!empty($row)) {
            return true;
        } else {
            return false;
        }

    }
    public function mudaStatus($order, $status, $mensagem)
    {
        if (!empty($status)) {
            $order->update_status($status, $mensagem.' - ');
        }
    }

    public static function callback_handler()
    {
        global $woocommerce;
        $instance = new WC_Gateway_iPag_Credito;
        $instance->writeLog('----- CALLBACK -----', true, 'callback');
        if (isset($_REQUEST['num_pedido'])) {
            $oid = !empty($_REQUEST['num_pedido']) ? $_REQUEST['num_pedido'] : '';
            $status = !empty($_REQUEST['status_pagamento']) ? $_REQUEST['status_pagamento'] : '';
            $trans_id = !empty($_REQUEST['id_transacao']) ? $_REQUEST['id_transacao'] : '';
            $trans_msg = !empty($_REQUEST['mensagem_transacao']) ? $_REQUEST['mensagem_transacao'] : '';
            $instance->writeLog(print_r($_REQUEST, true), false, 'callback');
        } else {
            $retorno = file_get_contents('php://input');
            $xml = simplexml_load_string($retorno, 'SimpleXMLElement');
            $oid = (int) $xml->num_pedido;

            $status = (int) $xml->status_pagamento;
            $trans_id = (string) $xml->id_transacao;
            $trans_msg = (string) $xml->mensagem_transacao;
            $instance->writeLog(print_r($xml, true), false, 'callback');
        }
        $status_2 = false;

        if (strpos($xml->num_pedido, 'a')) {
            $stats = get_post_meta($oid, '_status_payment_second_card', false);
            $status_2 = $stats[0];
        }
        if (strpos($xml->num_pedido, 'b')) {
            $stats = get_post_meta($oid, '_status_payment', false);
            $status_2 = $stats[0];
        }

        $statusPagamento = WC_iPag_Loader::verifyDoubleCardStatus($status, $status_2);

        $order = new WC_Order($oid);
        $order_id = is_callable(array($order, 'get_id')) ? $order->get_id() : $order->id;
        $subs = false;
        if (class_exists(WC_Subscriptions_Order)) {
            $subs = WC_Subscriptions_Order::order_contains_subscription($order);
        }

        if ($subs) {
            if ($status == 8) {
                if (!WC_iPag_Loader::existsTransaction($trans_id)) {
                    $registro = WC_iPag_Loader::registerTransaction($trans_id, $order_id);
                    WC_Subscriptions_Manager::process_subscription_payments_on_order($order);
                    update_post_meta($order_id, '_last_payment_sub', current_time('d/m/Y H:i:s'));
                }
            }
            if ($status == 3 || $status == 7) {
                WC_Subscriptions_Manager::process_subscription_payment_failure_on_order($order);
                $order->update_status('pending', __('Waiting Payment', 'ipag-gateway'));
            }
            update_post_meta($order_id, '_status_payment', $status);
        } else {
            update_post_meta($order_id, '_transaction_id', $trans_id);
            update_post_meta($order_id, '_status_payment', $$statusPagamento);
            update_post_meta($order_id, '_transaction_message', $trans_msg);
        }
        if($status_2) {
           $cartao = new WC_Gateway_iPag_CartaoDuplo;
        }
        else {
            $cartao = new WC_Gateway_iPag_Credito;
        }
        $status_aprovado = $cartao->get_option('status_aprovado');
        $status_reprovado = $cartao->get_option('status_reprovado');
        $status_capturado = $cartao->get_option('status_capturado');
        $status_cancelado = $cartao->get_option('status_cancelado');
        switch ($statusPagamento) {
            case 5: //aprovado
                WC_iPag_Loader::mudaStatus($order, $status_aprovado, WC_iPag_Loader::get_status((string) $statusPagamento));
                break;
            case 8: //aprovado e capturado
                WC_iPag_Loader::mudaStatus($order, $status_capturado, WC_iPag_Loader::get_status((string) $statusPagamento));
                break;
            case 3: //cancelado
                WC_iPag_Loader::mudaStatus($order, $status_cancelado, WC_iPag_Loader::get_status((string) $statusPagamento));
                break;
            case 7: //reprovado
                WC_iPag_Loader::mudaStatus($order, $status_reprovado, WC_iPag_Loader::get_status((string) $statusPagamento));
                break;
        }
        $order->add_order_note("Pedido atualizado. Status: ".WC_iPag_Loader::get_status($statusPagamento));

        wp_redirect(WC_iPag_Loader::get_return_url($order));
        $instance->writeLog('----- FIM DO CALLBACK -----', true, 'callback');
        exit();
    }

    public static function webhook_handler()
    {
        $instance = new WC_Gateway_iPag_Credito;
        $instance->writeLog('----- WEBHOOK -----', true, 'webhook');
        $callback = file_get_contents('php://input');
        $instance->writeLog($callback, true, 'webhook');

        if (file_exists(plugin_dir_path(__DIR__).'../ipag-woocommerce-webhook/webhook-handler.php')) {
            require_once plugin_dir_path(__FILE__).'../libs/autoload.php';
            require_once plugin_dir_path(__FILE__).'/support/Credentials.php';
            require_once plugin_dir_path(__FILE__).'/support/Environment.php';
            require_once plugin_dir_path(__FILE__).'/support/SplitRule.php';
            require_once plugin_dir_path(__FILE__).'/support/IpagHandler.php';
            require_once plugin_dir_path(__FILE__).'/support/Transaction.php';
            require_once plugin_dir_path(__FILE__).'/support/webhook-handler-interface.php';
            require_once plugin_dir_path(__DIR__).'../ipag-woocommerce-webhook/webhook-handler.php';

            $payload = json_decode($callback, true);

            $credentials = new Credentials($instance->identification, $instance->apikey);
            $ambiente = $instance->environment == 'test' ? Environment::SANDBOX : Environment::PRODUCTION;
            $environment = new Environment($ambiente);
            $transaction = new Transaction(
                $payload['id'],
                $payload['uuid'],
                $payload['attributes']['order_id'],
                $payload['attributes']['amount'],
                $payload['attributes']['status']['code'],
                $payload['attributes']['method'],
                $payload
            );

            $handler = new WebhookHandler($credentials, $environment, $transaction);
            echo $handler->execute();
        }

        exit();
    }

    protected function isAprovado($statusPagamento)
    {
        $ret = false;

        switch ($statusPagamento) {
            case 1: //1=iniciado
            case 2: //2=boleto impresso
            case 4: //4=em analise
            case 6: //6=aprovado valor parcial
            case 7: //7=nao autorizado
                $ret = false;
                break;
            case 5: //5=aprovado
            case 8: //8=aprovado e capturado
                $ret = true;
                break;
            default:
                $ret = false;
                break;
        }

        return ($ret);
    }

    protected function isCapturado($statusPagamento)
    {
        $ret = false;

        switch ($statusPagamento) {
            case 1: //1=iniciado
            case 2: //2=boleto impresso
            case 4: //4=em analise
            case 5: //5=aprovado
            case 6: //6=aprovado valor parcial
            case 7: //7=nao autorizado
                $ret = false;
                break;
            case 8: //8=aprovado e capturado
                $ret = true;
                break;
            default:
                $ret = false;
                break;
        }

        return ($ret);
    }

    protected function isReprovado($statusPagamento)
    {
        $ret = false;

        switch ($statusPagamento) {
            case 1: //1=iniciado
            case 2: //2=boleto impresso
            case 4: //4=em analise
            case 6: //6=aprovado valor parcial
            case 5: //5=aprovado
            case 8: //8=aprovado e capturado
                $ret = false;
                break;
            case 7: //7=nao autorizado
                $ret = true;
                break;
            default:
                $ret = false;
                break;
        }

        return ($ret);
    }

    protected function isCancelado($statusPagamento)
    {
        $ret = false;

        switch ($statusPagamento) {
            case 1: //1=iniciado
            case 2: //2=boleto impresso
            case 4: //4=em analise
            case 5: //5=aprovado
            case 6: //6=aprovado valor parcial
            case 7: //7=nao autorizado
            case 8: //8=aprovado e capturado
                $ret = false;
                break;
            case 3: //3=cancelados
                $ret = true;
                break;
            default:
                $ret = false;
                break;
        }

        return ($ret);
    }

    public static function consultTransaction()
    {
        $order_id = $_REQUEST['order'];
        $transid = $_REQUEST['transid'];
        $method = $_REQUEST['method'];
        $allowed_methods = array(
            'WC_Gateway_iPag_ItauShopline',
            'WC_Gateway_iPag_Credito',
            'WC_Gateway_iPag_Debito',
            'WC_Gateway_iPag_Boleto',
            'WC_Gateway_iPag_CartaoDuplo',
            'WC_Gateway_iPag_Pix',
        );

        if (in_array($method, $allowed_methods)) {
            $instance = new $method();
            $identificacao = $instance->identification;
            $apikey = $instance->apikey;
        } else {
            return false;
        }
        $instance->writeLog('----- CONSULTA-----', true, 'consult');

        if ($instance->environment == 'test') {
            $url = 'https://sandbox.ipag.com.br/';
        } else {
            $url = 'https://api.ipag.com.br/';
        }

        $url .= 'service/consult';
        $order = new WC_Order($order_id);

        $payload = array(
            'identificacao' => $identificacao,
            'transId'       => $transid,
            'url_retorno'   => 'XML',
        );
        $headers = array(
            'Authorization' => 'Basic '.base64_encode($identificacao.':'.$apikey),
        );
        $args = array(
            'body'    => $payload,
            'timeout' => 60,
            'headers' => $headers,
            'cookies' => array(),
        );
        $result = wp_remote_post($url, $args);

        $instance->writeLog($result['body'],false,'consult');

        $xml = simplexml_load_string($result['body'], 'SimpleXMLElement');
        //fazer update dos post_metas
        if (is_object($xml)) {
            if (strpos($xml->num_pedido, 'b')) {
                if ((string) $xml->status_pagamento == '8') {
                    update_post_meta($order_id, '_transaction_message_second_card', 'Transação Aprovada e Capturada');
                    $xml->mensagem_transacao = 'Transação Aprovada e Capturada';
                } else {
                    update_post_meta($order_id, '_transaction_message_second_card', (string) $xml->mensagem_transacao);
                }
                update_post_meta($order_id, '_status_payment_second_card', (string) $xml->status_pagamento);

            } else {
                if ((string) $xml->status_pagamento == '8') {
                    update_post_meta($order_id, '_transaction_message', 'Transação Aprovada e Capturada');
                    $xml->mensagem_transacao = 'Transação Aprovada e Capturada';
                } else {
                    update_post_meta($order_id, '_transaction_message', (string) $xml->mensagem_transacao);
                }
                update_post_meta($order_id, '_status_payment', (string) $xml->status_pagamento);
            }
            if (!$instance->existsTransaction((string) $xml->id_transacao)) {
                $registro = $instance->registerTransaction((string) $xml->id_transacao, $order_id, (string) $xml->status_pagamento);
            } else {
                $registro = $instance->updateTransaction((string) $xml->id_transacao, (string) $xml->status_pagamento);
            }
            $retorno = array('status' => (string) $xml->status_pagamento, 'mensagem' => (string) $xml->mensagem_transacao);

            $status_2 = false;
            $status = (string) $xml->status_pagamento;
            if (strpos($xml->num_pedido, 'a')) {
                $stats = get_post_meta($order_id, '_status_payment_second_card', false);
                $status_2 = $stats[0];
            }
            if (strpos($xml->num_pedido, 'b')) {
                $stats = get_post_meta($order_id, '_status_payment', false);
                $status_2 = $stats[0];
            }
            if($status_2) {
               $cartao = new WC_Gateway_iPag_CartaoDuplo;
            }
            else {
                $cartao = new WC_Gateway_iPag_Credito;
            }
            $status_aprovado = $cartao->get_option('status_aprovado');
            $status_reprovado = $cartao->get_option('status_reprovado');
            $status_capturado = $cartao->get_option('status_capturado');
            $status_cancelado = $cartao->get_option('status_cancelado');

            $statusPagamento = $instance->verifyDoubleCardStatus($status, $status_2);

            switch ($statusPagamento) {
                case 5: //aprovado
                    $instance->mudaStatus($order, $status_aprovado, $instance->get_status((string) $statusPagamento));
                    break;
                case 8: //aprovado e capturado
                    $instance->mudaStatus($order, $status_capturado, $instance->get_status((string) $statusPagamento));
                    break;
                case 3: //cancelado
                    $instance->mudaStatus($order, $status_cancelado, $instance->get_status((string) $statusPagamento));
                    break;
                case 7: //reprovado
                    $instance->mudaStatus($order, $status_reprovado, $instance->get_status((string) $statusPagamento));
                    break;
            }
        }
        echo json_encode($retorno);
        $instance->writeLog('----- FIM DA CONSULTA -----', true, 'consult');
        wp_die();
    }

    public static function verifyDoubleCardStatus($status1, $status2)
    {
        $statusPagamento = false;
        if ($status2 !== false) {
            $self = new WC_Gateway_iPag_CartaoDuplo();

            if ($self->isCapturado($status1)) {
                if ($self->isCapturado($status2)) {
                    $statusPagamento = 8;
                } else {
                    $statusPagamento = 6;
                }
            } elseif ($self->isAprovado($status1)) {
                if ($self->isAprovado($status2)) {
                    $statusPagamento = 5;
                } else {
                    $statusPagamento = 6;
                }
            } elseif ($self->isReprovado($status1)) {
                if ($self->isAprovado($status2)) {
                    $statusPagamento = 6;
                } else {
                    $statusPagamento = 7;
                }
            } elseif ($self->isCancelado($status1)) {
                if ($self->isCancelado($status2)) {
                    $statusPagamento = 3;
                } else {
                    $statusPagamento = 6;
                }
            } else {
                $statusPagamento = $status1;
            }
        }
        else {
            return $status1;
        }
        return $statusPagamento;
    }

    public static function consult_ajax()
    {
        global $theorder;
        if ($theorder) {
            $transid = get_post_meta($_REQUEST['post'], '_transaction_id', true);
            $transidSecond = get_post_meta($_REQUEST['post'], '_transaction_id_second_card', true);
            // WooCommerce 3.0 or later.
            if (method_exists($theorder, 'get_meta')) {
                $method = $theorder->get_payment_method();
            } else {
                $method = $theorder->payment_method;
            }
            if ($method == 'ipag-gateway_itaushopline') {
                $class_method = 'WC_Gateway_iPag_ItauShopline';
            }

            if ($method == 'ipag-gateway') {
                $class_method = 'WC_Gateway_iPag_Credito';
            }

            if ($method == 'ipag-gateway_debito') {
                $class_method = 'WC_Gateway_iPag_Debito';
            }

            if ($method == 'ipag-gateway_boleto') {
                $class_method = 'WC_Gateway_iPag_Boleto';
            }

            if ($method == 'ipag-gateway_pix') {
                $class_method = 'WC_Gateway_iPag_Pix';
            }

            if ($method == 'ipag-gateway-double-card') {
                $class_method = 'WC_Gateway_iPag_CartaoDuplo';
            }

            if ($class_method):
                $instance = new $class_method();
                ?>
                        <script type="text/javascript">
                            jQuery(document).ready(function() {
                                jQuery('#consult_transaction').on('click', consultTransaction);
                            });
                            function consultTransaction(){

                                jQuery('#consult_transaction').val('<?php _e('Consulting...', 'ipag-gateway')?>');
                                jQuery.ajax({
                                   method: 'POST',
                                   url: ajaxurl,
                                   data :{
                                       action: 'consultipag',
                                       order: '<?php echo $_REQUEST['post']; ?>',
                                       transid: '<?php echo $transid; ?>',
                                       method: '<?php echo $class_method; ?>',
                                   },
                                   success: function(response){
                                       response = JSON.parse(response);
                                       jQuery('#t_msg').html(response.mensagem);
                                       jQuery('#consult_transaction').val('<?php _e('Consulted', 'ipag-gateway')?>');
                                       jQuery('#consult_transaction').removeClass('button-primary');
                                       jQuery('#consult_transaction').attr('disabled', 'disabled');
                                       location.reload();
                                   }
                               });
                            }
                        </script>
                        <?php
    if (!empty($transidSecond)): ?>
                        <script type="text/javascript">
                            jQuery(document).ready(function() {
                                jQuery('#consult_transaction_second').on('click', consultTransaction_second);
                            });

                            function consultTransaction_second(){

                                jQuery('#consult_transaction_second').val('<?php _e('Consulting...', 'ipag-gateway')?>');
                                jQuery.ajax({
                                    method: 'POST',
                                    url: ajaxurl,
                                    data :{
                                        action: 'consultipag',
                                        order: '<?php echo $_REQUEST['post']; ?>',
                                        transid: '<?php echo $transidSecond; ?>',
                                        method: '<?php echo $class_method; ?>',
                                    },
                                    success: function(response) {
                                        response = JSON.parse(response);
                                        jQuery('#t_msg_second').html(response.mensagem);
                                        jQuery('#consult_transaction_second').val('<?php _e('Consulted', 'ipag-gateway')?>');
                                        jQuery('#consult_transaction_second').removeClass('button-primary');
                                        jQuery('#consult_transaction_second').attr('disabled', 'disabled');
                                        location.reload();
                                    }
                                });
                            }
                        </script>
                        ?>
                        <?php
endif;
            endif;
        }
    }

    public function maskPayload($payload)
    {
        $sReq = '';
        $maskedPayload = array();
        $sensibleData = array('num_cartao', 'cvv_cartao', 'num_cartao_2', 'cvv_cartao_2');
        foreach ($payload as $k => $v) {
            $value = $v;
            if (in_array($k, $sensibleData)) {
                $sub = substr($k, 0, 3);
                if ($sub == 'num') {
                    $value = preg_replace('/^(\d{6})(\d+)(\d{4})$/', '$1******$3', $value);
                } else {
                    $value = preg_replace('/\d/', '*', $value);
                }
            }
            $maskedPayload[$k] = $value;
        }
        return $maskedPayload;
    }

    public function pagsegurosession($psemail, $pstoken, $psambiente)
    {
        if (empty($psemail) || empty($pstoken)) {
            return;
        }

        $session_id = WC()->session->get('pagseguro_payment_session_id');
        if (empty($session_id)) {
            $ambiente = $psambiente; //0 = sandbox, 1 = produção

            $email = $psemail;
            $token = $pstoken;
            $param = "?email=".$email."&token=".$token;

            $url = "https://ws.sandbox.pagseguro.uol.com.br/v2/sessions".$param;
            $environment = 'sandbox';
            if ($ambiente) {
                $url = "https://ws.pagseguro.uol.com.br/v2/sessions".$param;
                $environment = 'production';
            }
            $opts = array('http' => array('method' => 'POST'));
            $context = stream_context_create($opts);
            // @$result = file_get_contents($url, false, $context, -1, 40000);
            $result = wp_remote_post($url);

            if (is_wp_error($result)) {
                $this->writeLog('Erro WP_Error: '.$result->get_error_message(), false, 'error');
            }
            elseif ($result && isset($result['body'])) {
                $xml = simplexml_load_string($result['body'], 'SimpleXMLElement', LIBXML_NOERROR);
                if ($xml === false) {
                    $this->writeLog("PagSeguro Não foi possível gerar o Session Id ");
                    return '';
                }
                $json = json_decode(json_encode((array) $xml), true);
                $id = array_shift($json);
                $session_id = $id;
                WC()->session->set('pagseguro_payment_session_id', $id);
            } else {
                $this->writeLog("PagSeguro Não foi possível gerar o Session Id ");
                $session_id = '';
            }
        }
        return $session_id;

    }

    public function getIpagSessionId($login, $key, $endpoint)
    {
        if (empty($login) || empty($key)) {
            return;
        }

        $url = $endpoint . '/service/sessionToken';
        $headers = array(
            'Authorization' => 'Basic '.base64_encode($login.':'.$key),
            'Content-Type'  => 'application/json'
        );
        $args = array(
            #'body'    => $payload,
            'timeout' => 60,
            'headers' => $headers,
            'cookies' => array(),
        );
        $result = wp_remote_post($url, $args);

        if (is_wp_error($result)) {
            $this->writeLog('Erro WP_Error: '.$result->get_error_message(), false, 'error');
            $session_id = '';
        }
        elseif ($result && isset($result['body'])) {
            $json = json_decode($result['body']);
            $session_id = $json->token;
        } else {
            $this->writeLog("iPag: Não foi possível gerar o SessionId ");
            $session_id = '';
        }
        return $session_id;
    }

    public static function reset_session()
    {
        WC()->session->__unset('pagseguro_payment_session_id');
    }

    public function getIpagLogo($class)
    {
        $option = get_option('woocommerce_ipag-gateway_settings');
        if ($option && array_key_exists('brand', $option) && $option['brand'] == 'yes') {
            return apply_filters('woocommerce_ipag_icon', plugins_url('images/ipag.png', plugin_dir_path(__FILE__)));
        }
        return false;
    }
}
