<?php

    class WC_Gateway_iPag_Credito extends WC_iPag_Loader
    {

        public function __construct()
        {

            $this->id = 'ipag-gateway';
            $this->has_fields = true;
            $this->method_title = __('iPag - Credit Card', 'ipag-gateway');
            $this->method_description = __('iPag Secure Payment', 'ipag-gateway');
            $this->supports = array('products', 'refunds', 'subscriptions');

            $this->init_form_fields();
            $this->init_settings();

            $this->enabled = $this->get_option('enabled');
            $this->title = $this->get_option('title');
            $this->identification = $this->get_option('identification');
            $this->apikey = $this->get_option('apikey');
            $this->accepted_cards = $this->get_option('accepted_cards');
            $this->pagseguro_habilitado = $this->get_option('psenabled');
            $this->pagseguro_ambiente = $this->get_option('psenvironment');
            $this->pagseguro_email = $this->get_option('pagseguro_email');
            $this->pagseguro_token = $this->get_option('pagseguro_token');
            $this->smallest_installment = $this->get_option('smallest_installment');
            $this->maximum_installment = $this->get_option('maximum_installment');
            $this->interest_rate = $this->get_option('interest_rate');
            $this->interest_free_installment = $this->get_option('interest_free_installment');
            $this->environment = $this->get_option('environment');
            $this->debug = $this->get_option('debug');
            $this->brand = $this->get_option('brand');
            $this->icon = $this->getIpagLogo($this);

            $this->status_aprovado = $this->get_option('status_aprovado');
            $this->status_reprovado = $this->get_option('status_reprovado');
            $this->status_capturado = $this->get_option('status_capturado');
            $this->status_cancelado = $this->get_option('status_cancelado');

            $this->prod_url = 'https://api.ipag.com.br/';
            $this->test_url = 'https://sandbox.ipag.com.br/';

            add_action('woocommerce_update_options_payment_gateways_'.$this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_admin_order_data_after_shipping_address', array($this, 'orderInfo'));
            add_action('wp_enqueue_scripts', array($this, 'checkout_scripts'));
            add_action('admin_footer', array($this, 'capture_ajax'));
        }

        public function init_form_fields()
        {
            $statuses = array('' => '-- Selecione um status --');
            $statuses = array_merge($statuses, wc_get_order_statuses());
            $this->form_fields = array(
                'enabled'                   => array(
                    'title'   => __('Enable iPag Payment Gateway', 'ipag-gateway'),
                    'type'    => 'checkbox',
                    'label'   => __('Enable', 'ipag-gateway'),
                    'default' => 'no',
                ),
                'title'                     => array(
                    'title'       => __('iPag Title', 'ipag-gateway'),
                    'type'        => 'text',
                    'description' => __('This controls the title that users will see during checkout.', 'ipag-gateway'),
                    'default'     => __('iPag - Credit Card', 'ipag-gateway'),
                    'desc_tip'    => true,
                ),
                'identification'            => array(
                    'title'       => __('Store Identification', 'ipag-gateway'),
                    'type'        => 'text',
                    'description' => __('Your store identification registered at iPag Panel.', 'ipag-gateway'),
                    'default'     => '',
                    'desc_tip'    => true,
                ),
                'apikey'                    => array(
                    'title'       => __('Ipag API KEY', 'ipag-gateway'),
                    'type'        => 'text',
                    'description' => __('Your store API KEY at iPag Panel.', 'ipag-gateway'),
                    'default'     => '',
                    'desc_tip'    => true,
                ),
                'accepted_cards'            => array(
                    'title'       => __('Accepted Card Brands', 'ipag-gateway'),
                    'type'        => 'multiselect',
                    'description' => __('Select the card brands that will be accepted for payment.', 'ipag-gateway'),
                    'desc_tip'    => true,
                    'class'       => 'wc-enhanced-select',
                    'default'     => array('visa', 'mastercard'),
                    'options'     => array(
                        'visa'       => 'Visa',
                        'mastercard' => 'MasterCard',
                        'diners'     => 'Diners',
                        'discover'   => 'Discover',
                        'elo'        => 'Elo',
                        'amex'       => 'American Express',
                        'hipercard'  => 'Hipercard',
                        'hiper'      => 'Hiper',
                        'jcb'        => 'JCB',
                        'aura'       => 'Aura',
                    ),
                ),
                'smallest_installment'      => array(
                    'title'       => __('Smallest Installment', 'ipag-gateway'),
                    'type'        => 'text',
                    'description' => __('Smallest value of each installment.', 'ipag-gateway'),
                    'desc_tip'    => true,
                    'default'     => '5.00',
                ),
                'maximum_installment'       => array(
                    'title'       => __('Installment Within', 'ipag-gateway'),
                    'type'        => 'select',
                    'description' => __('Maximum number of installments for orders in your store.', 'ipag-gateway'),
                    'desc_tip'    => true,
                    'class'       => 'wc-enhanced-select',
                    'default'     => '6',
                    'options'     => array(
                        '1'  => '1x',
                        '2'  => '2x',
                        '3'  => '3x',
                        '4'  => '4x',
                        '5'  => '5x',
                        '6'  => '6x',
                        '7'  => '7x',
                        '8'  => '8x',
                        '9'  => '9x',
                        '10' => '10x',
                        '11' => '11x',
                        '12' => '12x',
                    ),
                ),
                'interest_rate'             => array(
                    'title'       => __('Interest Rate(%)', 'ipag-gateway'),
                    'type'        => 'text',
                    'description' => __('Percentage of interest that will be charged to the customer in the installment.', 'ipag-gateway'),
                    'desc_tip'    => true,
                    'default'     => '1.49',
                ),
                'interest_free_installment' => array(
                    'title'       => __('Interest-Free Installment', 'ipag-gateway'),
                    'type'        => 'select',
                    'description' => __('Number of interest-free installments', 'ipag-gateway'),
                    'desc_tip'    => true,
                    'class'       => 'wc-enhanced-select',
                    'default'     => '6',
                    'options'     => array(
                        '1'  => '1x',
                        '2'  => '2x',
                        '3'  => '3x',
                        '4'  => '4x',
                        '5'  => '5x',
                        '6'  => '6x',
                        '7'  => '7x',
                        '8'  => '8x',
                        '9'  => '9x',
                        '10' => '10x',
                        '11' => '11x',
                        '12' => '12x',
                    ),
                ),
                'environment'               => array(
                    'title'    => __('Environment', 'ipag-gateway'),
                    'type'     => 'select',
                    'desc_tip' => false,
                    'default'  => 'production',
                    'options'  => array(
                        'test'       => __('Test', 'ipag-gateway'),
                        'production' => __('Production', 'ipag-gateway'),
                    ),
                ),
                'debug'                     => array(
                    'title'    => __('Enable Debug', 'ipag-gateway'),
                    'label'    => __('Enable', 'ipag-gateway'),
                    'type'     => 'checkbox',
                    'desc_tip' => false,
                    'default'  => 'no',
                ),
                'brand'                     => array(
                    'title'    => __('Enable iPag Brand', 'ipag-gateway'),
                    'label'    => __('Enable', 'ipag-gateway'),
                    'type'     => 'checkbox',
                    'desc_tip' => false,
                    'default'  => 'no',
                ),
                'status_aprovado'           => array(
                    'title'       => __('Status Aprovado', 'ipag-gateway'),
                    'type'        => 'select',
                    'desc_tip'    => true,
                    'description' => __('O pedido mudará automaticamente para este status em caso de aprovação no iPag', 'ipag-gateway'),
                    'default'     => 'wc-on-hold',
                    'options'     => $statuses,
                ),
                'status_reprovado'          => array(
                    'title'       => __('Status Reprovado', 'ipag-gateway'),
                    'type'        => 'select',
                    'desc_tip'    => true,
                    'description' => __('O pedido mudará automaticamente para este status em caso de reprova no iPag', 'ipag-gateway'),
                    'default'     => 'wc-failed',
                    'options'     => $statuses,
                ),
                'status_capturado'          => array(
                    'title'       => __('Status Capturado', 'ipag-gateway'),
                    'type'        => 'select',
                    'desc_tip'    => true,
                    'description' => __('O pedido mudará automaticamente para este status quando a transação for capturada no iPag', 'ipag-gateway'),
                    'default'     => 'wc-processing',
                    'options'     => $statuses,
                ),
                'status_cancelado'          => array(
                    'title'       => __('Status Cancelado', 'ipag-gateway'),
                    'type'        => 'select',
                    'desc_tip'    => true,
                    'description' => __('O pedido mudará automaticamente para este status quando a transação for cancelada no iPag', 'ipag-gateway'),
                    'default'     => 'wc-cancelled',
                    'options'     => $statuses,
                ),
                'psenabled'                 => array(
                    'title'   => __('Enable', 'ipag-gateway').' PagSeguro',
                    'type'    => 'checkbox',
                    'label'   => __('Enable', 'ipag-gateway'),
                    'default' => 'no',
                ),
                'psenvironment'             => array(
                    'title'    => 'PagSeguro '.__('Environment', 'ipag-gateway'),
                    'type'     => 'select',
                    'desc_tip' => false,
                    'default'  => 'production',
                    'options'  => array(
                        '0' => __('Test', 'ipag-gateway'),
                        '1' => __('Production', 'ipag-gateway'),
                    ),
                ),
                'pagseguro_email'           => array(
                    'title'       => __('PagSeguro Email', 'ipag-gateway'),
                    'type'        => 'text',
                    'desc_tip'    => true,
                    'description' => __('E-mail para integração com PagSeguro', 'ipag-gateway'),
                    'default'     => '',
                ),
                'pagseguro_token'           => array(
                    'title'       => __('PagSeguro Token', 'ipag-gateway'),
                    'type'        => 'text',
                    'desc_tip'    => true,
                    'description' => __('Token para integração com PagSeguro', 'ipag-gateway'),
                    'default'     => '',
                ),
            );
        }

        public function orderInfo()
        {
            $order_id = $_REQUEST['post'];
            $order = new WC_Order($order_id);
            $method = is_callable(array($order, 'get_payment_method')) ? $order->get_payment_method() : $order->payment_method;
            if ($method === 'ipag-gateway') {
                $this->order_info();
            }
        }

        public function payment_fields()
        {
            $total = WC()->cart->total;
            $juros = $this->interest_rate;
            $v_minimo = $this->smallest_installment;
            $maxparcelas = $this->maximum_installment;

            if (empty($total)) {
                $ped = new WC_Order(get_query_var('order-pay'));
                $total = $ped->get_total();
            }

            if ($total / $v_minimo < $maxparcelas) {
                $maxparcelas = (int) ($total / $v_minimo);
            }
            if ($this->environment == 'test') {
                $url = $this->test_url;
            } else {
                $url = $this->prod_url;
            }
            $params = array(
                'maximum_installment'       => $maxparcelas,
                'smallest_installment'      => $v_minimo,
                'interest_rate'             => $juros,
                'interest_free_installment' => $this->interest_free_installment,
                'parcelas_juros'            => $this->valorParcelas($total, $maxparcelas, $juros),
                'ipag_session_id'           => $this->getIpagSessionId($this->identification, $this->apikey, $url),
                'static_path'               => 'https://stc.pagseguro.uol.com.br',

                'pagseguro_enabled'         => $this->pagseguro_habilitado,
                'pagseguro_session'         => $this->pagsegurosession($this->pagseguro_email, $this->pagseguro_token, $this->pagseguro_ambiente),
                'total'                     => $total,
                'ano'                       => date('Y'),
                'accepted_cards'            => $this->accepted_cards,
                'ipag_test'                 => $this->environment == 'test' ? 'true' : 'false',
                'errors'                    => array(
                    'invalid_card'        => __('Invalid credit card number.', 'ipag-gateway'),
                    'invalid_cvv'         => __('Invalid CVV.', 'ipag-gateway'),
                    'invalid_name'        => __('Invalid name.', 'ipag-gateway'),
                    'invalid_expiry'      => __('Invalid expiry date.', 'ipag-gateway'),
                    'expired_card'        => __('Expired card.', 'ipag-gateway'),
                    'invalid_cpf'         => __('Invalid CPF.', 'ipag-gateway'),
                    'invalid_installment' => __('Please choose an installment option.', 'ipag-gateway'),
                ),
            );
            wc_get_template(
                'ipag-gateway-template-credito.php', $params, '', WC_iPag_Gateway::get_templates_path()
            );
        }

        public function checkout_scripts()
        {
            if (is_checkout() && $this->enabled == 'yes') {
                wp_enqueue_script('ipag-credito-js', 'https://api.ipag.com.br/js/dist/ipag.js', array(), null, true);
                wp_enqueue_script('ipag-formatter-js', 'https://cdnjs.cloudflare.com/ajax/libs/formatter.js/0.1.5/formatter.min.js', array(), null, true);
                wp_enqueue_script('ipag-crypto-js', 'https://cdnjs.cloudflare.com/ajax/libs/crypto-js/3.1.2/rollups/md5.js', array(), null, true);
                wp_enqueue_script('pagseguro-credito-js', 'https://stc.'.($this->pagseguro_ambiente ? '' : 'sandbox.').'pagseguro.uol.com.br/pagseguro/api/v2/checkout/pagseguro.directpayment.js', array(), null, true);
                wp_enqueue_script('ipag-cartao-js', plugins_url('js/cartao.js', dirname(__FILE__)), array(), null, true);

                wp_localize_script(
                    'ipag-cartao-js',
                    'errors',
                    array(
                        'invalid_card'        => __('Invalid credit card number.', 'ipag-gateway'),
                        'invalid_cvv'         => __('Invalid CVV.', 'ipag-gateway'),
                        'invalid_name'        => __('Invalid name.', 'ipag-gateway'),
                        'invalid_expiry'      => __('Invalid expiry date.', 'ipag-gateway'),
                        'expired_card'        => __('Expired card.', 'ipag-gateway'),
                        'invalid_cpf'         => __('Invalid CPF.', 'ipag-gateway'),
                        'invalid_installment' => __('Please choose an installment option.', 'ipag-gateway'),
                    )
                );
            }
        }

        public function expiryMesValido($mes, $ano)
        {
            $ano = substr(date('Y'), 0, 2).$ano;
            if (strlen($mes) == 2) {
                $intMes = intval($mes);
                $currentAno = intval(date('Y'));
                $ano = intval($ano);
                if ($ano > $currentAno) {
                    if ($intMes >= 1 && $intMes <= 12) {
                        return true;
                    }
                } elseif ($ano == $currentAno) {
                    $currentMes = intval(date('m'));
                    if ($intMes >= $currentMes) {
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            }
            return false;
        }

        public function expiryAnoValido($ano)
        {
            $valid = false;
            $ano = substr(date('Y'), 0, 2).$ano;

            if (strlen($ano) == 4) {
                $intAno = intval($ano);
                $currentAno = intval(date('Y'));
                if ($intAno >= $currentAno) {
                    return true;
                }
            }
            return $valid;
        }

        public function cartaoValido($number)
        {
            settype($number, 'string');
            $sumTable = array(
                array(0, 1, 2, 3, 4, 5, 6, 7, 8, 9),
                array(0, 2, 4, 6, 8, 1, 3, 5, 7, 9));
            $sum = 0;
            $flip = 0;
            for ($i = strlen($number) - 1; $i >= 0; $i--) {
                $sum += $sumTable[$flip++ & 0x1][$number[$i]];
            }
            return $sum % 10 === 0;
        }

        public function process_payment($order_id)
        {
            global $woocommerce;
            $this->writeLog('----- PAGAMENTO -----');
            $order = new WC_Order($order_id);
            $subs = false;
            if (class_exists('WC_Subscriptions_Order')) {
                $subs = wcs_order_contains_subscription($order);
                $sub_order = wcs_get_subscriptions_for_order($order);
                // Só é permitido uma recorrencia no carrinho
                $sub_order = array_pop($sub_order);
            }

            if ($subs) {
                $sub_signup_fee = $sub_order->get_sign_up_fee();
                $subs_period = $sub_order->billing_period;
                $subs_interval = $sub_order->billing_interval;
                $subs_length = wcs_estimate_periods_between($sub_order->get_time('start'), $sub_order->get_time('end'), $sub_order->billing_period);
                $subs_trial_period = $sub_order->trial_period;
                $subs_trial_length = wcs_estimate_periods_between($sub_order->get_time('start'), $sub_order->get_time('trial_end'), $sub_order->trial_period);

            }
            $expiry = (isset($_POST['ipag_credito_card_expiry'])) ? $_POST['ipag_credito_card_expiry'] : '';
            $expiry_split = explode('/', $expiry);
            $acquirerToken = (isset($_POST['cartao_tokenpagseguro'])) ? $_POST['cartao_tokenpagseguro'] : '';
            $fingerprint_pagseguro = (isset($_POST['cartao_hashpagseguro'])) ? $_POST['cartao_hashpagseguro'] : '';
            $cc_nome = (isset($_POST['ipag_credito_card_name'])) ? $_POST['ipag_credito_card_name'] : '';

            $cc_numero = (isset($_POST['ipag_credito_card_num'])) ? $_POST['ipag_credito_card_num'] : '';
            $cc_numero = preg_replace('/\s+/', '', $cc_numero);
            $cc_val_mes = (isset($expiry_split[0])) ? $expiry_split[0] : '';
            $cc_val_ano = (isset($expiry_split[1])) ? $expiry_split[1] : '';
            if (strlen($cc_val_mes) > strlen($cc_val_ano)) {
                $cc_val_mes = str_pad($cc_val_mes, 4, "0", STR_PAD_LEFT);
                $cc_val_ano = substr($cc_val_mes, 2, 2);
                $cc_val_mes = substr($cc_val_mes, 0, 2);
            }

            $this->writeLog('Datas: '.print_r($cc_val_mes, true).'/'.print_r($cc_val_ano, true));
            $cc_cvv = (isset($_POST['ipag_credito_card_cvv'])) ? $_POST['ipag_credito_card_cvv'] : '';
            $cc_parcelas = (isset($_POST['ipag_credito_installments'])) ? $_POST['ipag_credito_installments'] : 0;
            $cc_cpf = (isset($_POST['ipag_credito_card_cpf'])) ? $_POST['ipag_credito_card_cpf'] : '';
            $cc_cpf = preg_replace('/\D/', '', $cc_cpf);
            $ipagtoken = (isset($_POST['ipag_credito_card_token'])) ? $_POST['ipag_credito_card_token'] : '';

            $maxparcelas = $this->maximum_installment;
            $cc_metodo = $_POST['ipag_credito_card_type'];
            if (empty($cc_metodo)) {
                $cc_metodo = $this->getCardType($cc_numero);
            }

            if (($cc_parcelas + 1) > $this->interest_free_installment) {
                $total = $this->valorParcelas($order->get_total(), $maxparcelas, $this->interest_rate);
                $total = $total[$cc_parcelas] * ($cc_parcelas + 1);
            } else {
                $total = (float) $order->get_total();
            }
            $args = $this->getOrderData($order);
            $documento = empty($args['documento']) ? $cc_cpf : $args['documento'];
            $produtos = $this->getDescricaoPedido($order);

            $ano = date('Y');
            $mes = date('m');
            $ano2d = preg_replace('/^[\d]{2}/', '', $ano);

            $anovalid1 = true;
            if ($cc_val_ano < $ano2d) {
                $anovalid1 = false;
            }
            if ($cc_val_ano == $ano2d && $cc_val_mes < $mes) {
                $anovalid1 = false;
            }
            $fingerprintPS = (isset($_POST['cartao_hashpagseguro'])) ? $_POST['cartao_hashpagseguro'] : '';
            $cardTokenPS = (isset($_POST['cartao_tokenpagseguro'])) ? $_POST['cartao_tokenpagseguro'] : '';

            if ($this->pagseguro_habilitado == 'yes') {

                $this->writeLog('Hash PagSeguro Cartão '.print_r($_POST['cartao_hashpagseguro'], true));
                $this->writeLog('Token PagSeguro Cartão '.print_r($_POST['cartao_tokenpagseguro'], true));
                //check pagseguro enabled
                if (empty($_POST['cartao_hashpagseguro']) || empty($_POST['cartao_tokenpagseguro'])) {
                    $this->writeLog('Erro: Recuperação do token PagSeguro. ');
                    $error_message = __(' Não foi possível processar o cartão. Verifique os dados do cartão!', 'ipag-gateway');
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                    $fingerprintPS = '';
                    $cardTokenPS = '';
                    return array(
                        'result'   => 'fail',
                        'redirect' => '',
                    );
                } else {
                    $fingerprintPS = $_POST['cartao_hashpagseguro'];
                    $cardTokenPS = $_POST['cartao_tokenpagseguro'];
                }
            }

            $validPayload = in_array($cc_metodo, $this->accepted_cards) && $this->cartaoValido($cc_numero) && !empty($cc_nome) && strlen($cc_cvv) > 2 && !empty($cc_numero) && $anovalid1 && $this->expiryMesValido($cc_val_mes, $cc_val_ano);

            if ($validPayload || !empty($ipagtoken)) {
                $payload = array(
                    'identificacao'    => $this->identification,
                    'metodo'           => $cc_metodo,
                    'operacao'         => 'Pagamento',
                    'pedido'           => $order_id,
                    'valor'            => number_format($total, 2, '.', ''),
                    'parcelas'         => $cc_parcelas + 1,
                    'acquirerToken'    => $cardTokenPS,
                    'fingerprint'      => $fingerprintPS,
                    'cpf_cartao'       => $cc_cpf,
                    'nome'             => $args['billingName'],
                    'documento'        => preg_replace('/[\D]/', '', $documento),
                    'email'            => $args['billingEmail'],
                    'fone'             => preg_replace('/[\D]/', '', $args['billingPhone']),
                    'endereco'         => trim($args['billingAddress']),
                    'numero_endereco'  => str_replace(' ', '', $args['billingNumber']),
                    'bairro'           => empty($args['billingNeighborhood']) ? 'bairro' :
                    trim(preg_replace("/[^a-zA-Z ]/", "", $args['billingNeighborhood'])),
                    'complemento'      => empty($args['billingAddress2']) ? '' :
                    trim(preg_replace("/[^a-zA-Z ]/", "", $args['billingAddress2'])),
                    'cidade'           => $args['billingCity'],
                    'estado'           => substr($args['billingState'], 0, 2),
                    'pais'             => 'Brasil',
                    'cep'              => preg_replace('/[\D]/', '', $args['billingPostcode']),
                    'endereco_entrega'         => trim($args['shippingAddress']),
                    'numero_endereco_entrega'  => str_replace(' ', '', $args['shippingNumber']),
                    'bairro_entrega'           => empty($args['shippingNeighborhood']) ? 'bairro' :
                    trim(preg_replace("/[^a-zA-Z ]/", "", $args['shippingNeighborhood'])),
                    'complemento_entrega'      => empty($args['shippingAddress2']) ? '' :
                    trim(preg_replace("/[^a-zA-Z ]/", "", $args['shippingAddress2'])),
                    'cidade_entrega'           => $args['shippingCity'],
                    'estado_entrega'           => substr($args['shippingState'], 0, 2),
                    'cep_entrega'              => preg_replace('/[\D]/', '', $args['shippingPostcode']),
                    'descricao_pedido' => $produtos,
                    'retorno_tipo'     => 'XML',
                    'url_retorno'      => home_url('/wc-api/wc_gateway_ipag/?id='.$order_id),
                    'ip'               => $this->getClientIp($this->pagseguro_habilitado == 'yes'),
                    'vencto'           => '',
                );
                if (empty($ipagtoken)) {
                    $payload['nome_cartao'] = $cc_nome;
                    $payload['num_cartao'] = $cc_numero;
                    $payload['cvv_cartao'] = $cc_cvv;
                    $payload['mes_cartao'] = $cc_val_mes;
                    $payload['ano_cartao'] = $cc_val_ano;
                } else {
                    $payload['token_cartao'] = $ipagtoken;
                }

                $maskedPayload = $this->maskPayload($payload);
                $this->writeLog(print_r($maskedPayload, true));
                if ($subs) {
                    $inicio = $sub_order->get_time('next_payment');
                    $next_payment = $sub_order->get_time('start');
                    $payload['valor_rec'] = number_format($sub_order->get_total(), 2, '.', '');
                    $payload['frequencia'] = $subs_interval;
                    $payload['intervalo'] = $subs_period;
                    $payload['inicio'] = date('d/m/Y', $inicio);
                    if ($subs_length != 0) {
                        $payload['ciclos'] = (int) $subs_length;
                    }

                    if ($subs_trial_length) {
                        $payload['trial'] = ($sub_signup_fee > 0) ? '0' : '1';
                        // $inicio = strtotime("+".$subs_trial_length." ".$subs_period, $inicio);
                        // $payload['inicio'] = date('d/m/Y', $inicio);
                        //$payload['trial_frequencia'] = $subs_period;
                        //$payload['trial_ciclos'] = $subs_trial_length;
                    } else {
                        $payload['trial'] = '0';
                    }
                }

                $visitorid = '';
                if (isset($_COOKIE['_kdt'])) {
                    $json = str_replace("\\", "", $_COOKIE['_kdt']);
                    $cookie = json_decode($json, true);
                    $visitorid = $cookie['i'];
                    $payload['visitor_id'] = $visitorid;
                }

                $success = $this->processCreditCard($payload, $order);

                if ($success) {
                    $order->set_total($total);
                    $woocommerce->cart->empty_cart();
                    $this->writeLog('----- FIM DO PAGAMENTO -----');
                    return array(
                        'result'   => 'success',
                        'redirect' => $this->get_return_url($order),
                    );
                } else {
                    $this->writeLog('----- FIM DO PAGAMENTO -----');
                    return array(
                        'result'   => 'fail',
                        'redirect' => '',
                    );
                }
            } else {
                if (!$this->cartaoValido($cc_numero)) {
                    $error_message = __(' Invalid credit card. Please choose another one!', 'ipag-gateway');
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                } elseif (!$anovalid1 || !$this->expiryMesValido($cc_val_mes, $cc_val_ano)) {
                    $error_message = __(' Validade do cartão expirada! ', 'ipag-gateway');
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                } elseif (empty($cc_numero)) {
                    $error_message = __(' Preencha o número do cartão !', 'ipag-gateway');
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                } elseif (empty($cc_nome)) {
                    $error_message = __(' Preencha o nome do cartão!', 'ipag-gateway');
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                } elseif (strlen($cc_cvv) < 3) {
                    $error_message = __(' Preencha o CVV do cartão corretamente!', 'ipag-gateway');
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                } elseif ($cc_val_ano == $ano && $cc_val_mes < $mes) {
                    $error_message = __(' Validade do cartão expirada!', 'ipag-gateway');
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                } else {
                    $error_message = __(' This card isn\'t accepted. Please choose another one!', 'ipag-gateway');
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                }
                update_post_meta($order_id, '_transaction_message', "Cartão inválido. Pagamento não enviado ao gateway.");
                $order->add_order_note("Cartão inválido. Pagamento não enviado ao gateway.");
                $this->writeLog('Cartão inválido. Pagamento não enviado ao gateway.');
            }
            $this->writeLog('----- FIM DO PAGAMENTO -----');
        }

        public function processCreditCard($payload, $order)
        {
            global $woocommerce;
            header('Content-type: text/html; charset=UTF-8');
            if ($this->environment == 'test') {
                $url = $this->test_url;
            } else {
                $url = $this->prod_url;
            }

            $url .= 'service/payment';
            $fields = '';
            $fields_log = '';

            foreach ($payload as $key => $value) {
                if ($key == 'num_cartao') {
                    $value = preg_replace('/^(\d{6})(\d+)(\d{4})$/', '$1******$3', $value);
                } elseif ($key == 'cvv_cartao') {
                    $value = preg_replace('/\d/', '*', $value);
                }
                $fields_log .= $key.'='.$value.'&';
                rtrim($fields_log, '&');
            }
            $this->writeLog('URL: '.$url.'?'.$fields_log.PHP_EOL);

            $headers = array(
                'Authorization' => 'Basic '.base64_encode($this->identification.':'.$this->apikey),
            );
            $args = array(
                'body'    => $payload,
                'timeout' => 60,
                'headers' => $headers,
                'cookies' => array(),
            );
            $result = wp_remote_post($url, $args);

            $this->writeLog($result['body']);
            try {
                $xml = simplexml_load_string($result['body'], 'SimpleXMLElement');
            } catch (Exception $e) {
                $xml = '';
                $this->writeLog($e);
            }
            $order_id = is_callable(array($order, 'get_id')) ? $order->get_id() : $order->id;
            if (is_object($xml)) {
                switch ($xml->status_pagamento) {
                    case 5: //aprovado
                        $this->mudaStatus($order, $this->status_aprovado, $this->get_status((string) $xml->status_pagamento));
                        break;
                    case 8: //aprovado e capturado
                        $this->mudaStatus($order, $this->status_capturado, $this->get_status((string) $xml->status_pagamento));
                        break;
                    case 3: //cancelado
                        $this->mudaStatus($order, $this->status_cancelado, $this->get_status((string) $xml->status_pagamento));
                        break;
                    case 7: //reprovado
                        $this->mudaStatus($order, $this->status_reprovado, $this->get_status((string) $xml->status_pagamento));
                        break;
                }
                if ($xml->status_pagamento == 5 || $xml->status_pagamento == 8) {
                    version_compare($woocommerce->version, '3.0', ">=") ? wc_reduce_stock_levels($order_id) : $order->reduce_order_stock();
                }
                if(array_key_exists('token_cartao', $payload)) {
                    $expiry_split = explode('/', $xml->cartao->vencimento);
                    $cc_val_mes = (isset($expiry_split[0])) ? $expiry_split[0] : '';
                    $cc_val_ano = (isset($expiry_split[1])) ? $expiry_split[1] : '';
                    $payload['nome_cartao'] = $xml->cartao->titular;
                    $payload['num_cartao'] = $xml->cartao->numero;
                    $payload['mes_cartao'] = $cc_val_mes;
                    $payload['ano_cartao'] = $cc_val_ano;
                }
                if ($xml->status_pagamento == 7 || $xml->status_pagamento == 3) {
                    $error_message = (string) $xml->mensagem_transacao;
                    wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                    $card = $this->splitCard($payload['num_cartao']);
                    update_post_meta($order_id, '_card_type', (string) $this->cardTypeClearSale($payload['metodo']));
                    update_post_meta($order_id, '_card_bin', (string) $card['bin']);
                    update_post_meta($order_id, '_card_end', (string) $card['end']);
                    update_post_meta($order_id, '_card_masked', (string) $card['masked']);
                    update_post_meta($order_id, '_card_exp_month', (string) $payload['mes_cartao']);
                    update_post_meta($order_id, '_card_exp_year', (string) $payload['ano_cartao']);
                    update_post_meta($order_id, '_card_name', (string) $payload['nome_cartao']);
                    update_post_meta($order_id, '_card_cpf', (string) $payload['cpf_cartao']);
                    update_post_meta($order_id, '_transaction_id', (string) $xml->id_transacao, false);
                    update_post_meta($order_id, '_transaction_message', (string) $xml->mensagem_transacao, false);
                    update_post_meta($order_id, '_operator_message', (string) $xml->operadora_mensagem, false);
                    update_post_meta($order_id, '_installment_number', $payload['parcelas'].'x - Total: R$ '.$payload['valor'], false);
                    update_post_meta($order_id, '_status_payment', (string) $xml->status_pagamento);
                    //$order->update_status('failed', $this->get_status((string)$xml->status_pagamento) . ' - ');

                    $this->writeLog('ID  : '.(string) $xml->id_transacao.PHP_EOL.
                        'Status: '.(string) $xml->status_pagamento.' - '.$this->get_status((string) $xml->status_pagamento).PHP_EOL.
                        'Erro: '.(string) $xml->mensagem_transacao.PHP_EOL);

                    return true;
                } else {
                    $this->writeLog("Payment Complete. Order #".$order_id);
                    //$order->payment_complete();
                    $card = $this->splitCard($payload['num_cartao']);
                    update_post_meta($order_id, '_card_type', (string) $this->cardTypeClearSale($payload['metodo']));
                    update_post_meta($order_id, '_card_bin', (string) $card['bin']);
                    update_post_meta($order_id, '_card_end', (string) $card['end']);
                    update_post_meta($order_id, '_card_masked', (string) $card['masked']);
                    update_post_meta($order_id, '_card_exp_month', (string) $payload['mes_cartao']);
                    update_post_meta($order_id, '_card_exp_year', (string) $payload['ano_cartao']);
                    update_post_meta($order_id, '_card_name', (string) $payload['nome_cartao']);
                    update_post_meta($order_id, '_card_cpf', (string) $payload['cpf_cartao']);
                    update_post_meta($order_id, '_transaction_id', (string) $xml->id_transacao);
                    update_post_meta($order_id, '_transaction_message', (string) $xml->mensagem_transacao);
                    update_post_meta($order_id, '_operator_message', (string) $xml->operadora_mensagem);
                    update_post_meta($order_id, '_installment_number', $payload['parcelas'].'x - Total: R$ '.$payload['valor']);
                    update_post_meta($order_id, '_status_payment', (string) $xml->status_pagamento);
                    $this->registerTransaction((string) $xml->id_transacao, $order_id, (string) $xml->status_pagamento);
                    return true;
                }
            } else {
                $error_message = __(' Error processing payment. Check card data!', 'ipag-gateway');
                wc_add_notice(__('Payment error:', 'ipag-gateway').$error_message, 'error');
                $order->update_status('failed', __('Invalid card data', 'ipag-gateway'));
                $this->writeLog('Payment Error. Order #'.$order_id);
                return true;
            }
        }

        /**
         * Process a refund if supported.
         *
         * @param  int    $order_id Order ID.
         * @param  float  $amount Refund amount.
         * @param  string $reason Refund reason.
         * @return bool|WP_Error
         */
        public function process_refund($order_id, $amount = null, $reason = '')
        {
            $order = wc_get_order($order_id);
            $order_id = $order->get_id();
            $tid = get_post_meta($order_id, '_transaction_id', true);
            $status = get_post_meta($order_id, '_transaction_message', true);
            if ($amount < $order->get_total()) {
                return new WP_Error('error', __('Somente é possível realizar reembolso do valor total do pedido.', 'woocommerce'));
            }

            WC_iPag_Loader::writeLog('----- CANCELAMENTO-----', true);
            $id = $this->identification;
            $ambiente = $this->environment;
            $status_cancelado = $this->status_cancelado;

            if ($ambiente == 'test') {
                $url = 'https://sandbox.ipag.com.br/';
            } else {
                $url = 'https://api.ipag.com.br/';
            }

            $url .= 'service/cancel';
            $order = new WC_Order($order_id);

            $payload = array(
                'identificacao' => $id,
                'transId'       => $tid,
                'url_retorno'   => home_url('/wc-api/wc_gateway_ipag/?'),
                'tipo_retorno'  => 'XML',
            );
            $headers = array(
                'Authorization' => 'Basic '.base64_encode($this->identification.':'.$this->apikey),
            );
            $args = array(
                'body'    => $payload,
                'timeout' => 60,
                'headers' => $headers,
                'cookies' => array(),
            );
            $result = wp_remote_post($url, $args);

            WC_iPag_Loader::writeLog(print_r($result['body'], true), true);

            $xml = simplexml_load_string($result['body'], 'SimpleXMLElement');
            //fazer update dos post_metas
            if (is_object($xml)) {
                switch ($xml->status_pagamento) {
                    case 3: //aprovado e capturado
                        WC_iPag_Loader::mudaStatus($order, $status_cancelado, WC_iPag_Loader::get_status((string) $xml->status_pagamento));
                        break;
                }
                update_post_meta($order_id, '_transaction_message', (string) $xml->mensagem_transacao);
                update_post_meta($order_id, '_status_payment', (string) $xml->status_pagamento);
                $order->add_order_note(WC_iPag_Loader::get_status((string) $xml->status_pagamento).' - '.(string) $xml->mensagem_transacao);
                if (!WC_iPag_Loader::existsTransaction((string) $xml->id_transacao)) {
                    $registro = WC_iPag_Loader::registerTransaction((string) $xml->id_transacao, $order_id, (string) $xml->status_pagamento);
                } else {
                    $registro = WC_iPag_Loader::updateTransaction((string) $xml->id_transacao, (string) $xml->status_pagamento);
                }
                $retorno = array('status' => (string) $xml->status_pagamento, 'mensagem' => (string) $xml->mensagem_transacao);
            }
            WC_iPag_Loader::writeLog('----- FIM DO CANCELAMENTO -----', true);
            return isset($error) ? new WP_Error('error', 'Resposta iPag: '.$error) : true;
        }

        public function capturePayment()
        {
            WC_iPag_Loader::writeLog('----- CAPTURA-----', true);
            $order_id = $_REQUEST['order'];
            $transid = $_REQUEST['transid'];
            $id = $_REQUEST['id'];
            $ambiente = $_REQUEST['environment'];
            $status_capturado = $_REQUEST['status_capturado'];

            if ($ambiente == 'test') {
                $url = 'https://sandbox.ipag.com.br/';
            } else {
                $url = 'https://api.ipag.com.br/';
            }

            $url .= 'service/capture';
            $order = new WC_Order($order_id);

            $payload = array(
                'identificacao' => $id,
                'transId'       => $transid,
                'url_retorno'   => home_url('/wc-api/wc_gateway_ipag/?'),
                'tipo_retorno'  => 'XML',
            );

            $card = new WC_Gateway_iPag_Credito();
            $headers = array(
                'Authorization' => 'Basic '.base64_encode($card->identification.":".$card->apikey),
            );
            $args = array(
                'body'    => $payload,
                'timeout' => 60,
                'headers' => $headers,
                'cookies' => array(),
            );
            $result = wp_remote_post($url, $args);
            WC_iPag_Loader::writeLog($result['body'], true);

            $xml = simplexml_load_string($result['body'], 'SimpleXMLElement');
            //fazer update dos post_metas
            if (is_object($xml)) {
                switch ($xml->status_pagamento) {
                    case 8: //aprovado e capturado
                        WC_iPag_Loader::mudaStatus($order, $status_capturado, WC_iPag_Loader::get_status((string) $xml->status_pagamento));
                        break;
                }
                update_post_meta($order_id, '_transaction_message', (string) $xml->mensagem_transacao);
                update_post_meta($order_id, '_status_payment', (string) $xml->status_pagamento);
                $order->add_order_note(WC_iPag_Loader::get_status((string) $xml->status_pagamento).' - '.(string) $xml->mensagem_transacao);
                if (!WC_iPag_Loader::existsTransaction((string) $xml->id_transacao)) {
                    $registro = WC_iPag_Loader::registerTransaction((string) $xml->id_transacao, $order_id, (string) $xml->status_pagamento);
                } else {
                    $registro = WC_iPag_Loader::updateTransaction((string) $xml->id_transacao, (string) $xml->status_pagamento);
                }
                $retorno = array('status' => (string) $xml->status_pagamento, 'mensagem' => (string) $xml->mensagem_transacao);
            }
            echo json_encode($retorno);
            WC_iPag_Loader::writeLog('----- FIM DA CAPTURA -----', true);
            wp_die();
        }

        public function splitCard($card_number)
        {
            $card_len = strlen($card_number) - 10;
            preg_match('/[\d]{4}$/', $card_number, $card_end);
            $card_end = $card_end[0];
            preg_match('/^[\d]{6}/', $card_number, $card_bin);
            $card_bin = $card_bin[0];
            $masked = '';
            $masked .= $card_bin;
            for ($i = 0; $i < $card_len; $i++) {
                $masked .= '*';
            }

            $masked .= $card_end;

            return $retorno = array(
                'masked' => $masked,
                'bin'    => $card_bin,
                'end'    => $card_end,
            );
        }

        public function cardTypeClearSale($card_type)
        {
            $card = array(
                'mastercard' => 2,
                'visa'       => 3,
                'amex'       => 5,
                'discover'   => 4,
                'diners'     => 1,
                'elo'        => 4,
                'hipercard'  => 6,
                'hiper'      => 6,
            );
            return $card[$card_type];
        }

        public function admin_options()
        {
            echo '<h2>'._e('iPag Payment Gateway - Credit Card', 'ipag-gateway').'</h2>';
            echo '<table class="form-table">';
            $this->generate_settings_html();
            echo '</table>';
        }

        public function capture_ajax()
        {
            global $theorder;
            if ($theorder) {
                $transid = get_post_meta($_REQUEST['post'], '_transaction_id', true);
            ?>
            <script type="text/javascript">
                jQuery(document).ready(function() {
                    jQuery('#capt_payment').on('click', capturePayment);
                });
                function capturePayment(){
                    jQuery('#capt_payment').val('<?php _e('Capturing...', 'ipag-gateway')?>');
                    jQuery.ajax({
                       method: 'POST',
                       url: ajaxurl,
                       data :{
                           action: 'capture',
                           order: '<?php echo $_REQUEST['post']; ?>',
                           transid: '<?php echo $transid; ?>',
                           id: '<?php echo $this->identification; ?>',
                           environment: '<?php echo $this->environment; ?>',
                           status_capturado: '<?php echo $this->status_capturado; ?>',
                       },
                       success: function(response){
                           response = JSON.parse(response);
                           jQuery('#t_msg').html(response.mensagem);
                           jQuery('#capt_payment').val('<?php _e('Captured', 'ipag-gateway')?>');
                           jQuery('#capt_payment').removeClass('button-primary');
                           jQuery('#capt_payment').attr('disabled', 'disabled');
                       }
                    });
                }
            </script><?php
            }
        }
    }